@@ -1,6 +1,11 @@
 #!/usr/bin/python3
 # SPDX-License-Identifier: MIT
 
+"""
+This module contains the s0i3 prerequisite validator for amd-debug-tools.
+"""
+
+import configparser
 import logging
 import os
 import platform
@@ -9,13 +14,16 @@ import shutil
 import subprocess
 import tempfile
 import struct
-import pyudev
 from datetime import datetime
 from packaging import version
 
+import pyudev
+
 from amd_debug.wake import WakeIRQ
+from amd_debug.display import Display
 from amd_debug.kernel import get_kernel_log, SystemdLogger, DmesgLogger
 from amd_debug.common import (
+    apply_prefix_wrapper,
     BIT,
     clear_temporary_message,
     get_distro,
@@ -111,12 +119,37 @@ class PrerequisiteValidator(AmdTool):
         self.irqs = []
         self.smu_version = ""
         self.smu_program = ""
+        self.display = Display()
 
     def capture_once(self):
         """Capture the prerequisites once"""
         if not self.db.get_last_prereq_ts():
             self.run()
 
+    def capture_edid(self):
+        """Capture and decode the EDID data"""
+        edids = self.display.get_edid()
+        if len(edids) == 0:
+            self.db.record_debug("No EDID data found")
+            return True
+
+        for name, p in edids.items():
+            try:
+                cmd = ["edid-decode", p]
+                output = subprocess.check_output(cmd, stderr=subprocess.DEVNULL).decode(
+                    "utf-8"
+                )
+            except FileNotFoundError:
+                self.db.record_prereq(
+                    "edid-decode not installed, unable to decode EDID", "👀"
+                )
+                return True
+            except subprocess.CalledProcessError as e:
+                self.db.record_prereq(f"Failed to capture EDID table: {e.output}", "👀")
+                return False
+            self.db.record_debug(apply_prefix_wrapper(f"EDID for {name}:", output))
+        return True
+
     def check_amdgpu(self):
         """Check for the AMDGPU driver"""
         for device in self.pyudev.list_devices(subsystem="pci"):
@@ -167,10 +200,10 @@ class PrerequisiteValidator(AmdTool):
         if self.kernel_log.match_pattern("ath11k_pci.*wcn6855"):
             match = self.kernel_log.match_pattern("ath11k_pci.*fw_version")
             if match:
-                self.db.record_debug("WCN6855 version string: %s" % match)
+                self.db.record_debug(f"WCN6855 version string: {match}")
                 objects = match.split()
-                for i in range(0, len(objects)):
-                    if objects[i] == "fw_build_id":
+                for i, obj in enumerate(objects):
+                    if obj == "fw_build_id":
                         wcn6855 = objects[i + 1]
 
         if wcn6855:
@@ -346,6 +379,7 @@ class PrerequisiteValidator(AmdTool):
 
     def check_usb3(self):
         """Check for the USB4 controller"""
+        slots = []
         for device in self.pyudev.list_devices(subsystem="pci", PCI_CLASS="C0330"):
             slot = device.properties["PCI_SLOT_NAME"]
             if device.properties.get("DRIVER") != "xhci_hcd":
@@ -354,18 +388,27 @@ class PrerequisiteValidator(AmdTool):
                 )
                 self.failures += [MissingXhciHcd()]
                 return False
-            self.db.record_prereq(f"USB3 driver `xhci_hcd` bound to {slot}", "✅")
+            slots += [slot]
+        if slots:
+            self.db.record_prereq(
+                f"USB3 driver `xhci_hcd` bound to {', '.join(slots)}", "✅"
+            )
         return True
 
     def check_usb4(self):
         """Check if the thunderbolt driver is loaded"""
+        slots = []
         for device in self.pyudev.list_devices(subsystem="pci", PCI_CLASS="C0340"):
             slot = device.properties["PCI_SLOT_NAME"]
             if device.properties.get("DRIVER") != "thunderbolt":
                 self.db.record_prereq("USB4 driver `thunderbolt` missing", "❌")
                 self.failures += [MissingThunderbolt()]
                 return False
-            self.db.record_prereq(f"USB4 driver `thunderbolt` bound to {slot}", "✅")
+            slots += [slot]
+        if slots:
+            self.db.record_prereq(
+                f"USB4 driver `thunderbolt` bound to {', '.join(slots)}", "✅"
+            )
         return True
 
     def check_sleep_mode(self):
@@ -478,8 +521,8 @@ class PrerequisiteValidator(AmdTool):
                 continue
             if self.cpu_family and self.cpu_model and self.cpu_model_string:
                 self.db.record_prereq(
-                    "%s (family %x model %x)"
-                    % (self.cpu_model_string, self.cpu_family, self.cpu_model),
+                    f"{self.cpu_model_string} "
+                    f"(family {self.cpu_family:x} model {self.cpu_model:x})",
                     "💻",
                 )
                 break
@@ -487,7 +530,7 @@ class PrerequisiteValidator(AmdTool):
 
     # See https://github.com/torvalds/linux/commit/ec6c0503190417abf8b8f8e3e955ae583a4e50d4
     def check_fadt(self):
-        """Check the kernel emitted a message specific to 6.0 or later indicating FADT had a bit set."""
+        """Check the kernel emitted a message indicating FADT had a bit set."""
         found = False
         if not self.kernel_log:
             message = "Unable to test FADT from kernel log"
@@ -575,10 +618,10 @@ class PrerequisiteValidator(AmdTool):
         """Check if the user has permissions to write to /sys/power/state"""
         p = os.path.join("/", "sys", "power", "state")
         try:
-            with open(p, "w") as w:
+            with open(p, "w", encoding="utf-8") as _w:
                 pass
         except PermissionError:
-            self.db.record_prereq("%s" % Headers.RootError, "👀")
+            self.db.record_prereq(f"{Headers.RootError}", "👀")
             return False
         except FileNotFoundError:
             self.db.record_prereq("Kernel doesn't support power management", "❌")
@@ -608,7 +651,8 @@ class PrerequisiteValidator(AmdTool):
                     )
         if show_warning:
             self.db.record_prereq(
-                "Timer based wakeup doesn't work properly for your ASIC/firmware, please manually wake the system",
+                "Timer based wakeup doesn't work properly for your "
+                "ASIC/firmware, please manually wake the system",
                 "🚦",
             )
         return True
@@ -631,7 +675,7 @@ class PrerequisiteValidator(AmdTool):
                 for line in contents.split("\n"):
                     if "WAKE_INT_MASTER_REG:" in line:
                         val = "en" if int(line.split()[1], 16) & BIT(15) else "dis"
-                        self.db.record_debug("Windows GPIO 0 debounce: %sabled" % val)
+                        self.db.record_debug(f"Windows GPIO 0 debounce: {val}abled")
                         continue
                     if not header and re.search("trigger", line):
                         debug_str += line + "\n"
@@ -671,7 +715,9 @@ class PrerequisiteValidator(AmdTool):
                         self.db.record_prereq(f"{interface} has WoL enabled", "✅")
                     else:
                         self.db.record_prereq(
-                            f"Platform may have low hardware sleep residency with Wake-on-lan disabled. Run `ethtool -s {interface} wol g` to enable it if necessary.",
+                            "Platform may have low hardware sleep residency "
+                            "with Wake-on-lan disabled. Run `ethtool -s "
+                            f"{interface} wol g` to enable it if necessary.",
                             "🚦",
                         )
         return True
@@ -726,7 +772,8 @@ class PrerequisiteValidator(AmdTool):
             # Dictionary of instance id to firmware version mappings that
             # have been "reported" to be problematic
             device_map = {
-                "8c36f7ee-cc11-4a36-b090-6363f54ecac2": "0.1.26",  # https://gitlab.freedesktop.org/drm/amd/-/issues/3443
+                # https://gitlab.freedesktop.org/drm/amd/-/issues/3443
+                "8c36f7ee-cc11-4a36-b090-6363f54ecac2": "0.1.26",
             }
             interesting_plugins = ["nvme", "tpm", "uefi_capsule"]
             if device.get_plugin() in interesting_plugins:
@@ -743,7 +790,8 @@ class PrerequisiteValidator(AmdTool):
                     item in device.get_guids() or item in device.get_instance_ids()
                 ) and ver in device.get_version():
                     self.db.record_prereq(
-                        f"Platform may have problems resuming.  Upgrade the firmware for '{device.get_name()}' if you have problems.",
+                        "Platform may have problems resuming.  Upgrade the "
+                        f"firmware for '{device.get_name()}' if you have problems.",
                         "🚦",
                     )
         return True
@@ -792,12 +840,13 @@ class PrerequisiteValidator(AmdTool):
             else:
                 acpi_hid = ""
             # set prefix if last device
-            prefix = "| " if dev != devices[-1] else "└─"
-            debug_str += "{prefix}{name} [{acpi_hid}] : {acpi_path}\n".format(
-                prefix=prefix, name=name, acpi_hid=acpi_hid, acpi_path=acpi_path
-            )
+            prefix = "│ " if dev != devices[-1] else "└─"
+            debug_str += f"{prefix}{name} [{acpi_hid}] : {acpi_path}\n"
             if "IDEA5002" in name:
-                remediation = f"echo {parent.sys_path.split('/')[-1]} | sudo tee /sys/bus/i2c/drivers/{parent.driver}/unbind"
+                remediation = (
+                    f"echo {parent.sys_path.split('/')[-1]} | "
+                    f"sudo tee /sys/bus/i2c/drivers/{parent.driver}/unbind"
+                )
 
                 self.db.record_prereq(f"{name} may cause spurious wakeups", "❌")
                 self.failures += [I2CHidBug(name, remediation)]
@@ -834,22 +883,13 @@ class PrerequisiteValidator(AmdTool):
             if os.path.exists(p):
                 acpi = read_file(p)
                 debug_str += (
-                    "{prefix}{pci_slot_name} : {vendor} {cls} [{id}] : {acpi}\n".format(
-                        prefix=prefix,
-                        pci_slot_name=pci_slot_name,
-                        vendor=database_vendor,
-                        cls=database_class,
-                        id=pci_id,
-                        acpi=acpi,
-                    )
+                    f"{prefix}{pci_slot_name} : "
+                    f"{database_vendor} {database_class} [{pci_id}] : {acpi}\n"
                 )
             else:
-                debug_str += "{prefix}{pci_slot_name} : {vendor} {cls} [{id}]\n".format(
-                    prefix=prefix,
-                    vendor=database_vendor,
-                    pci_slot_name=pci_slot_name,
-                    cls=database_class,
-                    id=pci_id,
+                debug_str += (
+                    f"{prefix}{pci_slot_name} : "
+                    f"{database_vendor} {database_class} [{pci_id}]\n"
                 )
         if debug_str:
             self.db.record_debug(debug_str)
@@ -908,7 +948,7 @@ class PrerequisiteValidator(AmdTool):
                         stdout=subprocess.DEVNULL,
                         stderr=subprocess.DEVNULL,
                     )
-                    self.db.record_debug_file("%s.dsl" % prefix)
+                    self.db.record_debug_file(f"{prefix}.dsl")
                 except subprocess.CalledProcessError as e:
                     self.db.record_prereq(
                         f"Failed to capture ACPI table: {e.output}", "👀"
@@ -925,10 +965,10 @@ class PrerequisiteValidator(AmdTool):
             self.db.record_prereq(desc, "🔋")
 
     def capture_logind(self):
+        """Capture logind.conf settings"""
         base = os.path.join("/", "etc", "systemd", "logind.conf")
         if not os.path.exists(base):
             return True
-        import configparser
 
         config = configparser.ConfigParser()
         config.read(base)
@@ -982,7 +1022,8 @@ class PrerequisiteValidator(AmdTool):
             _, cpu_count, _, _ = read_cpuid(0, 0x80000026, 1)
             if cpu_count > max_cpus:
                 self.db.record_prereq(
-                    f"The kernel has been limited to {max_cpus} CPU cores, but the system has {cpu_count} cores",
+                    f"The kernel has been limited to {max_cpus} CPU cores, "
+                    f"but the system has {cpu_count} cores",
                     "❌",
                 )
                 self.failures += [LimitedCores(cpu_count, max_cpus)]
@@ -996,12 +1037,6 @@ class PrerequisiteValidator(AmdTool):
         except PermissionError:
             self.db.record_prereq("CPUID checks unavailable", "🚦")
 
-        if valid:
-            self.db.record_prereq(
-                f"{self.cpu_model_string} (family {self.cpu_family:x} model {self.cpu_model:x})",
-                "✅",
-            )
-
         return True
 
     def check_msr(self):
@@ -1020,7 +1055,7 @@ class PrerequisiteValidator(AmdTool):
                 if not check_bits(val, expect_val):
                     self.failures += [MSRFailure()]
                     return False
-                self.db.record_prereq("PC6 and CC6 enabled", "✅")
+            self.db.record_prereq("PC6 and CC6 enabled", "✅")
         except FileNotFoundError:
             self.db.record_prereq(
                 "Unable to check MSRs: MSR kernel module not loaded", "❌"
@@ -1111,7 +1146,9 @@ class PrerequisiteValidator(AmdTool):
         if "pcie_port_pm=off" in cmdline:
             return True
         self.db.record_prereq(
-            "Platform may hang resuming.  Upgrade your firmware or add pcie_port_pm=off to kernel command line if you have problems.",
+            "Platform may hang resuming.  "
+            "Upgrade your firmware or add pcie_port_pm=off to kernel command "
+            "line if you have problems.",
             "🚦",
         )
         return False
@@ -1139,6 +1176,7 @@ class PrerequisiteValidator(AmdTool):
             self.capture_linux_firmware,
             self.capture_logind,
             self.capture_pci_acpi,
+            self.capture_edid,
         ]
         checks = []
 
